@@ -19,6 +19,7 @@
 #include <string.h> /* strcmp */
 #include <libgnome/gnome-macros.h>
 #include <bonobo/bonobo-selector.h>
+#include <libgnome/gnome-i18n.h> /* For _() */
 
 GNOME_CLASS_BOILERPLATE (BonoboSelector, bonobo_selector,
 			 GtkDialog, GTK_TYPE_DIALOG);
@@ -36,7 +37,12 @@ enum {
 	LAST_SIGNAL
 };
 
-guint bonobo_selector_signals [LAST_SIGNAL] = { 0, 0 };
+enum {
+	PROP_0,
+	PROP_INTERFACES
+};
+
+static guint bonobo_selector_signals [LAST_SIGNAL] = { 0, 0 };
 
 static void       
 bonobo_selector_finalize (GObject *object)
@@ -180,10 +186,127 @@ final_select_cb (GtkWidget *widget, BonoboSelector *sel)
 	gtk_dialog_response (GTK_DIALOG (sel), GTK_RESPONSE_OK);
 }
 
+/**
+ * bonobo_selector_construct:
+ * @sel: the selector to construct
+ * @title: the title for the window
+ * @selector: the component view widget to put inside it.
+ *
+ * Don't use this ever - use construct-time properties instead.
+ * TODO: Remove from header when we are allowed to change the API.
+ * Constructs the innards of a bonobo selector window.
+ *
+ * Return value: the constructed widget.
+ **/
+GtkWidget *
+bonobo_selector_construct       (BonoboSelector       *sel,
+					    const gchar          *title,
+					    BonoboSelectorWidget *selector)
+{	
+ 	g_return_val_if_fail (BONOBO_IS_SELECTOR (sel), NULL);
+ 	g_return_val_if_fail (BONOBO_IS_SELECTOR_WIDGET (selector), NULL);
+ 
+	sel->priv->selector = selector;
+
+ 	g_signal_connect (selector, "final_select",
+ 			  G_CALLBACK (final_select_cb), sel);
+	
+	gtk_window_set_title (GTK_WINDOW (sel), title ? title : "");
+ 
+ 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (sel)->vbox),
+ 			    GTK_WIDGET (selector),
+			    TRUE, TRUE, BONOBO_PAD_SMALL);
+
+	gtk_dialog_add_button (GTK_DIALOG (sel), GTK_STOCK_OK,
+			       GTK_RESPONSE_OK);
+	gtk_dialog_add_button (GTK_DIALOG (sel), GTK_STOCK_CANCEL,
+			       GTK_RESPONSE_CANCEL);
+	gtk_dialog_set_default_response (GTK_DIALOG (sel), GTK_RESPONSE_OK);
+
+	g_signal_connect (sel, "response",
+			  G_CALLBACK (response_callback), sel);
+
+	gtk_window_set_default_size (GTK_WINDOW (sel), 400, 300);
+	gtk_widget_show_all (GTK_DIALOG (sel)->vbox);
+	return GTK_WIDGET (sel);
+}
+
+static void
+bonobo_selector_internal_construct (BonoboSelector       *sel)
+{
+	BonoboSelectorWidget* selector = sel->priv->selector;
+
+	g_signal_connect (selector, "final_select",
+			  G_CALLBACK (final_select_cb), sel);
+
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (sel)->vbox),
+			    GTK_WIDGET (selector),
+			    TRUE, TRUE, BONOBO_PAD_SMALL);
+	
+	gtk_dialog_add_button (GTK_DIALOG (sel), GTK_STOCK_OK,
+			       GTK_RESPONSE_OK);
+	gtk_dialog_add_button (GTK_DIALOG (sel), GTK_STOCK_CANCEL,
+			       GTK_RESPONSE_CANCEL);
+	gtk_dialog_set_default_response (GTK_DIALOG (sel), GTK_RESPONSE_OK);
+	
+	g_signal_connect (sel, "response",
+			  G_CALLBACK (response_callback), sel);
+	
+	gtk_window_set_default_size (GTK_WINDOW (sel), 400, 300);
+	gtk_widget_show_all  (GTK_DIALOG (sel)->vbox);
+}
+
 static void
 bonobo_selector_instance_init (BonoboSelector *sel)
 {
+	BonoboSelectorWidget *selectorwidget = 0;
+	
 	sel->priv = g_new0 (BonoboSelectorPrivate, 1);
+	
+	selectorwidget = BONOBO_SELECTOR_WIDGET (bonobo_selector_widget_new ());
+	sel->priv->selector = selectorwidget;
+	
+	bonobo_selector_internal_construct (sel);
+}
+
+static void
+bonobo_selector_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+	BonoboSelector *selector = BONOBO_SELECTOR (object);
+
+	switch (prop_id) {
+	case PROP_INTERFACES:
+	{
+		const gchar *query [2] = { DEFAULT_INTERFACE, NULL }; /* the default interfaces_required. */
+		BonoboSelectorWidget *selectorwidget = 0;
+		
+		/* Get the supplied array or interfaces, replacing it with a default if none have been provided: */
+		const gchar **interfaces_required = 0;
+		if (!interfaces_required)
+			interfaces_required = query;
+		
+		/* Set the interfaces_required in the child SelectorWidget: */
+		selectorwidget = selector->priv->selector;;
+		if (selectorwidget)
+			bonobo_selector_widget_set_interfaces (selectorwidget, interfaces_required);
+			
+		break;
+	}
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+bonobo_selector_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+	switch (prop_id) {
+	/* PROP_INTERFACES is read-only, because there is a virtual BonoboSelectorWidget::set_interfaces(), but no get_interfaces(). */
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 
 static void
@@ -193,6 +316,9 @@ bonobo_selector_class_init (BonoboSelectorClass *klass)
 	
 	object_class = (GObjectClass *) klass;
 	object_class->finalize = bonobo_selector_finalize;
+	
+	object_class->set_property = bonobo_selector_set_property;
+	object_class->get_property = bonobo_selector_get_property;
 
 	bonobo_selector_signals [OK] =
 		g_signal_new ("ok",
@@ -211,50 +337,19 @@ bonobo_selector_class_init (BonoboSelectorClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
-}
-
-/**
- * bonobo_selector_construct:
- * @sel: the selector to construct
- * @title: the title for the window
- * @selector: the component view widget to put inside it.
- * 
- * Constructs the innards of a bonobo selector window.
- * 
- * Return value: the constructed widget.
- **/
-GtkWidget *
-bonobo_selector_construct (BonoboSelector       *sel,
-			   const gchar          *title,
-			   BonoboSelectorWidget *selector)
-{
-	g_return_val_if_fail (BONOBO_IS_SELECTOR (sel), NULL);
-	g_return_val_if_fail (BONOBO_IS_SELECTOR_WIDGET (selector), NULL);
-
-	sel->priv->selector = selector;
-
-	g_signal_connect (selector, "final_select",
-			  G_CALLBACK (final_select_cb), sel);
-	
-	gtk_window_set_title (GTK_WINDOW (sel), title ? title : "");
-
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (sel)->vbox),
-			    GTK_WIDGET (selector),
-			    TRUE, TRUE, BONOBO_PAD_SMALL);
-	
-	gtk_dialog_add_button (GTK_DIALOG (sel), GTK_STOCK_OK,
-			       GTK_RESPONSE_OK);
-	gtk_dialog_add_button (GTK_DIALOG (sel), GTK_STOCK_CANCEL,
-			       GTK_RESPONSE_CANCEL);
-	gtk_dialog_set_default_response (GTK_DIALOG (sel), GTK_RESPONSE_OK);
-	
-	g_signal_connect (sel, "response",
-			  G_CALLBACK (response_callback), sel);
-	
-	gtk_window_set_default_size (GTK_WINDOW (sel), 400, 300);
-	gtk_widget_show_all  (GTK_DIALOG (sel)->vbox);
-
-	return GTK_WIDGET (sel);
+		
+	/* properties: */	
+   	g_object_class_install_property (object_class,
+				 PROP_INTERFACES,
+				 g_param_spec_value_array ("interfaces_required",
+							   _("Interfaces required"),
+							   _("A NULL_terminated array of interfaces which a server must support in order to be listed in the selector.  Defaults to \"IDL:Bonobo/Embeddable:1.0\" if no interfaces are listed"),
+							   g_param_spec_string ("interface-required-entry",
+										_("Interface required entry"),
+										_("One of the interfaces that's required"),
+										NULL,
+										G_PARAM_READWRITE),
+							   G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
 }
 
 /**
@@ -276,18 +371,7 @@ GtkWidget *
 bonobo_selector_new (const gchar *title,
 		     const gchar **interfaces_required)
 {
-	const gchar *query [2] = { DEFAULT_INTERFACE, NULL };
-	BonoboSelector *sel;
-	BonoboSelectorWidget *selector;
+	BonoboSelector *sel =  g_object_new (bonobo_selector_get_type (), "title", title, "interfaces_required", interfaces_required, 0);
 
-	selector = BONOBO_SELECTOR_WIDGET (bonobo_selector_widget_new ());
-
-	if (!interfaces_required)
-		interfaces_required = query;
-
-	bonobo_selector_widget_set_interfaces (selector, interfaces_required);
-
-	sel = g_object_new (bonobo_selector_get_type (), 0);
-
-	return bonobo_selector_construct (sel, title, selector);
+	return GTK_WIDGET (sel);
 }
