@@ -67,7 +67,7 @@ enum {
 	LAST_SIGNAL
 };
 
-guint signals [LAST_SIGNAL] = { 0 };
+static guint signals [LAST_SIGNAL] = { 0 };
 
 struct _BonoboUIEnginePrivate {
 	GObject      *view;
