@@ -34,6 +34,9 @@
 
 static GObjectClass *parent_class = NULL;
 
+static GQuark dockitem_id = 0;
+static GQuark toolitem_id = 0;
+
 #define PARENT_TYPE bonobo_ui_sync_get_type ()
 
 static GdkPixbuf *
@@ -620,11 +623,6 @@ create_dockitem (BonoboUISyncToolbar *sync,
 	gboolean can_config = TRUE;
 	BonoboUIToolbar *toolbar;
 
-	if ((prop = bonobo_ui_node_peek_attr (node, "behavior"))) {
-		if (!strcmp (prop, "detachable"))
-			force_detachable = TRUE;
-	}
-
 	if ((prop = bonobo_ui_node_peek_attr (node, "behavior"))) {
 		behavior_array = g_strsplit (prop, ",", -1);
 	
@@ -737,6 +735,7 @@ impl_bonobo_ui_sync_toolbar_update_root (BonoboUISync *sync,
 	const char *txt;
 	const char *dockname;
 	gboolean    tooltips;
+	gboolean    detachable;
 	BonoboDockItem *item;
 	BonoboUIToolbar *toolbar;
 	BonoboUIToolbarStyle look;
@@ -750,7 +749,14 @@ impl_bonobo_ui_sync_toolbar_update_root (BonoboUISync *sync,
 	if (!item)
 		item = create_dockitem (BONOBO_UI_SYNC_TOOLBAR (sync),
 					node, dockname);
-	
+
+	if ((txt = bonobo_ui_node_peek_attr (node, "behavior")) &&
+	    strstr (txt, "detachable"))
+		detachable = TRUE;
+	else
+		detachable = bonobo_ui_preferences_get_toolbar_detachable ();
+	bonobo_dock_item_set_locked (item, !detachable);
+
 	toolbar = BONOBO_UI_TOOLBAR (GTK_BIN (item)->child);
 
 	bonobo_ui_engine_stamp_root (sync->engine, node, GTK_WIDGET (toolbar));
@@ -809,7 +815,13 @@ static gboolean
 impl_bonobo_ui_sync_toolbar_can_handle (BonoboUISync *sync,
 					BonoboUINode *node)
 {
-	return bonobo_ui_node_has_name (node, "dockitem");
+	if (!dockitem_id) {
+		dockitem_id = g_quark_from_static_string ("dockitem");
+		toolitem_id = g_quark_from_static_string ("toolitem");
+	}
+
+	return (node->name_id == dockitem_id ||
+		node->name_id == toolitem_id);
 }
 
 static void
